-include_lib("eNet/include/eNet.hrl").

%% @type version(). HTTP version as a tuple, i.e. `{0, 9} | {1, 0} | {1, 1}'.
-type version() :: {0, 9} | {1, 0} | {1, 1}.

-export_type([version/0]).

-define(DefWsOpts, [
   binary
   , {packet, 4}
   , {active, false}
   , {reuseaddr, true}
   , {nodelay, false}
   , {delay_send, true}
   , {send_timeout, 15000}
   , {keepalive, true}
   , {exit_on_close, true}
   , {back_log, 1024}
]).

-export_type([wsOpt/0]).
-type wsOpt() ::
   listenOpt() |
   {wsMod, module()}.

-record(wsReq, {
   method :: method(),
   path :: binary(),
   version :: wsHttp:version(),
   scheme :: undefined | binary(),
   host :: undefined | binary(),
   port :: undefined | 1..65535,
   args :: [{binary(), any()}],
   headers :: headers(),
   body = <<>> :: body()
}).

-export_type([wsReq/0, method/0, body/0, headers/0, response_code/0]).

%% @type req(). A record representing an HTTP request.
-type wsReq() :: #wsReq{}.

%% @type http_method(). An uppercase atom representing a known HTTP verb or a
%% binary for other verbs.
-type method() :: 'OPTIONS' | 'GET' | 'HEAD' | 'POST'| 'PUT' | 'DELETE' | 'TRACE' | binary().

%% @type body(). A binary or iolist.
-type body() :: binary() | iolist().

-type header() :: {Key :: binary(), Value :: binary() | string()}.
-type headers() :: [header()].

-type response_code() :: 100..999.


-define(CONTENT_LENGTH_HEADER, 'Content-Length').
-define(EXPECT_HEADER, <<"expect">>).
-define(CONNECTION_HEADER, 'Connection').
-define(TRANSFER_ENCODING_HEADER, <<"Transfer-Encoding">>).

-type sendfile_opts() :: [{chunk_size, non_neg_integer()}].