-module(wsEgHandover).
-export([init/2, handle/2, handle_event/3]).

-include("wsCom.hrl").
-behaviour(wsHer).

%% @doc Return `{ok, handover}' if `Req''s path is `/hello/world',
%% otherwise `ignore'.
init(Req, _Args) ->
   case wsReq:path(Req) of
      [<<"hello">>, <<"world">>] ->
         {ok, handover};
      _ ->
         ignore
   end.

%% TODO: write docstring
-spec handle(Req, Args) -> Result when
   Req :: elli:wsReq(),
   Args :: wsHer:callback_args(),
   Result :: wsHer:result().
handle(Req, Args) ->
   handle(wsReq:method(Req), wsReq:path(Req), Req, Args).


handle('GET', [<<"hello">>, <<"world">>], Req, _Args) ->
   Body = <<"Hello World!">>,
   Size = integer_to_binary(size(Body)),
   Headers = [{"Connection", "close"}, {"Content-Length", Size}],
   wsHttp:send_response(Req, 200, Headers, Body),
   {close, <<>>};


handle('GET', [<<"hello">>], Req, _Args) ->
   %% Fetch a GET argument from the URL.
   Name = wsReq:get_arg(<<"name">>, Req, <<"undefined">>),
   {ok, [], <<"Hello ", Name/binary>>}.


%% @hidden
handle_event(_, _, _) ->
   ok.
